{{
Author: Bob Sweeney
         
Main program listing 
Display menu with update of user settings

DAT contains menu text
MenuX - references menu text for each menu level
MenuXvalue - identifies if menu item has a value, reference number
             is the user[x] value to display. if 0 no 
MenuNum - contains # menu elements for each MenuX item
}}

'********************************
CON
'********************************
  _CLKMODE      = XTAL1 + PLL8X                        
  _XINFREQ      = 8_000_000
  up            = 18                                    'pins for buttons
  down          = 19
  select        = 20
'  i2cadd        = $7FE0                                 'allows for 32 saved variables

'***************************************
'  System Definitions      
'***************************************

  _OUTPUT       = 1             'Sets pin to output in DIRA register
  _INPUT        = 0             'Sets pin to input in DIRA register  
  _HIGH         = 1             'High=ON=1=3.3v DC
  _ON           = 1
  _LOW          = 0             'Low=OFF=0=0v DC
  _OFF          = 0
  _ENABLE       = 1             'Enable (turn on) function/mode
  _DISABLE      = 0             'Disable (turn off) function/mode

'***************************************
'  GUI Definitions
'***************************************
  'screen sizing constants for 256 color mode of uOLED-96-PROP                                       
  _xpixels      = 96                                    'Screen width
  _ypixels      = 64                                    'Screen height          
  _pixelperlong = 4                                     'Each tile requires 4 longs
  
  _xtiles       = _xpixels/_pixelperlong                'Each tile is 4 pixels x 4 pixels
  _ytiles       = _ypixels/_pixelperlong                                        
  _screensize   = (_xtiles * _ytiles * _pixelperlong)   'Size needed for arrays

'***************************************
'  Color Definitions
'***************************************

  _red          = %11100000
  _green        = %00011100
  _blue         = %00000011
  _yellow       = %11111100
  _purple       = %11100011
  _turq         = %00011111
  _white        = %11111111
  _black        = %00000000
  _ltbrown      = %01101000
  _dkblue       = %00000001
  _dkpurple     = %00100001
  _magenta      = %01100001
  _ltviolet     = %10100011
  _dkgreen      = %00000100
  _dkred        = %01000000

'********************************
VAR
'********************************
  long MemDisp[_screensize]                                                     'OLED display driver variables
  long MemWork[_screensize]                                                     'graphics driver variables
  long p, digits, exponent, integer, tens, zeros,  precision
  long positive_chr, decimal_chr, thousands_chr, thousandths_chr
  long p1,p2,p3
  long P1Low,P2Low,P3Low,P1Hi,P2Hi,P3Hi,P1sf,P2sf,P3sf
  word menustack[20]
  byte upflag,downflag,selectflag, cog, cog1
  byte float_string[20]

'********************************
OBJ
'********************************
  OLED     : "Brilldea-8bitDisplay-Driver-Ver006.spin"                     '8-bit per pixel display driver
  Graphics : "Brilldea-8bitGraphics-Driver-Ver000.spin"                    '8-bit per pixel graphics driver
  FMath    : "Float32mod"
  Num      : "Numbersmod"
  Serial   : "FullDuplexSerialPlus"

'********************************
PUB Main | ok, value1
'initialize display and enter menu loop
'********************************
  \OLED.start
  OLED.reset
  OLED.initialize(@MemDisp)
  
  'Start the graphics driver and it up for operation
  Graphics.setup(_xtiles, _ytiles, @MemDisp)
  Graphics.clear
  Num.Init
  FMath.start
  SetPrecision(3)
  P1sf      := 195                                                         'scaling factor cell1 - set in cal, save as integer
  P2sf      := 190                                                         'scaling factor cell2 - set in cal, save as integer
  P3sf      := 205                                                         'scaling factor cell3 - set in cal, save as integer
  Serial.start(21, -1, 3, 2400)
  ok := (cog1 := cognew(GetButton, @menustack)+1) > 0
  repeat
    TestDisplay

'********************************
PUB GetButton
'monitor for button presses - set appropriate flags
'********************************
  dira[up] := 0
  dira[down] := 0
  dira[select] := 0
  repeat
    oled.pauseMSec(50)
    if ina[up] == 1                                                             'up
      upflag := 1
    if ina[down] == 1                                                             'down
      downflag := 1
    if ina[select] == 1                                                             'Select
      selectflag := 1  

'********************************
PUB GetPOValues | x, c1, c2, c3, y
'  get cell PO2
'********************************
  repeat while x <> 1
    x := serial.getDec 
  c1 := serial.getDec
  c2 := serial.getDec
  c3 := serial.getDec
  y := (c1*100)/P1sf
  p1 := FMath.FDiv(FMath.FFloat(y),100.0)  
  y := (c2*100)/P2sf
  p2 := FMath.FDiv(FMath.FFloat(y),100.0)  
  y := (c3*100)/P3sf
  p3 := FMath.FDiv(FMath.FFloat(y),100.0)

'********************************
PUB TestDisplay | x, c1, c2, c3, i1, i2, i3, avepo
'********************************
  Graphics.plotText(0,7,0, _yellow, string("Test"))
  repeat                                                                        'repeat for entire DAT file
    Graphics.clear
    Graphics.plotLine(0,10,95,10, _blue)
    GetPOValues 
    avePO := GetCellAve(p1,p2,p3)                                               'get average PO

    'Cell1 logic
    if CellHi(p1)                                                               'if cell > hi or < lo limit, color rectangle red
      Graphics.PlotRectangle(0,0,31,7, _red)
    elseif CellLo(p1)                                                           'if cell > hi or < lo limit, color rectangle red
      Graphics.PlotRectangle(0,0,31,7, _red)
    Graphics.plotText(0,0,0, _yellow, FloatToFormat(p1, 4, 2))

    'Cell2 logic
    if CellHi(p2) 
      Graphics.PlotRectangle(32,0,64,7, _red)
    elseif CellLo(p2)
      Graphics.PlotRectangle(32,0,64,7, _red)
    Graphics.plotText(6,0,0, _yellow, FloatToFormat(p2, 4, 2))

    'Cell3 logic
    if CellHi(p3) 
      Graphics.PlotRectangle(64,0,95,7, _red)
    elseif CellLo(p3)
      Graphics.PlotRectangle(64,0,95,7, _red)
    Graphics.plotText(12,0,0, _yellow, FloatToFormat(p3, 4, 2))

    'print average PO
    Graphics.PutLText(FloatToFormat(AvePO, 4, 2), 0,1,1, _yellow)
    if CellHi(AvePO)
      Graphics.PutLText(string("<"), 5,1,1, _red)
    elseif CellLo(AvePO)
      Graphics.PutLText(string(";"),5,1,1, _red)
    if x == 0
      x := 1
      Graphics.PlotRectangle(32,56,38,63, _yellow)
    else
      x := 0
      Graphics.PlotRectangle(32,56,38,63, _black)

    Graphics.plotText(6,7,0, _yellow, Num.ToStr(clkfreq, NUM#DEC))
    Graphics.copy(@MemDisp)
    OLED.screenUpdate(1,@MemDisp)
    oled.pauseMSec(1000)

      
'***********************************************************************************************************************
'  Cell math routines
'***********************************************************************************************************************

'********************************
PUB GetCellAve(cell1, cell2, cell3)| x
'********************************
  x := FMath.FAdd(cell1, cell2)
  x := FMath.FAdd(x, cell3)
  Result := FMath.FDiv(x,3.0)                                'get average PO2

'********************************
PUB CellHi(cell)
'returns true if cell value > maxPO value
'********************************
  case Fmath.FCmp(cell, 1.4)
    -1: Result := false
    0: Result := true
    1: Result := true

'********************************
PUB CellLo(cell)
'returns true if cell value < minPO value
'********************************
  case Fmath.FCmp(cell, 0.8)
    -1: Result := true
    0: Result := true
    1: Result := false

'********************************
PUB Debug(x,y)
'********************************
  Graphics.PlotRectangle(0,55,65,63,_black)
  Graphics.plotText(0,7,0, _yellow, Num.ToStr(x,Num#DEC))
  Graphics.plotText(6,7,0, _yellow, Num.ToStr(y,Num#DEC))
  Graphics.copy(@MemDisp)
  oled.pauseMSec(2000)

'********************************
PUB Debugstr(x)
'********************************
  Graphics.PlotRectangle(0,55,65,63,_black)
  Graphics.plotText(0,7,0, _yellow, x)
  Graphics.copy(@MemDisp)
  oled.pauseMSec(2000)

'********************************
PUB DebugFP(x,y)
'********************************
  Graphics.PlotRectangle(0,55,65,63,_black)
  Graphics.plotText(0,7,0, _yellow, FloatToFormat(x, 4, 3))
  Graphics.plotText(6,7,0, _yellow, FloatToFormat(y, 4, 3))
  Graphics.copy(@MemDisp)
  oled.pauseMSec(2000)
  
'***********************************************************************************************************************
' FloatToString Routines - copied due to Brillo driver requirements
'***********************************************************************************************************************

'********************************
PUB FloatToString(Single) : StringPtr
'********************************

  'perform initial setup
  StringPtr := Setup(Single)

  'eliminate trailing zeros
  if integer
    repeat until integer // 10
      integer /= 10
      tens /= 10
      digits--
  else
    digits~

  'express number according to exponent
  case exponent
    'in range left of decimal
    11..0:
      AddDigits(exponent + 1)
    'in range right of decimal
    -1..digits - 13:
      zeros := -exponent
      AddDigits(1)
{    'out of range, do scientific notation
    other:
      DoScientific}

  'terminate z-string
  byte[p]~

'********************************
PUB FloatToFormat(single, width, dp) : stringptr | n, w2
'********************************

  ' get string pointer
  stringptr := p := @float_string

  ' width must be 1 to 9, dp must be 0 to width-1
  w2 := width  :=  width #> 1 <# 9
  dp := dp #> 0 <# (width - 2)
  if dp > 0
    w2--
  if single & $8000_0000 or positive_chr
    w2--

  ' get positive scaled integer value
  n := FMath.FRound(FMath.FMul(single & $7FFF_FFFF , FMath.FFloat(teni[dp])))

  if n => teni[w2]
    ' if format error, display asterisks
    repeat while width
      if --width == dp
        if decimal_chr
          byte[p++] := decimal_chr
        else
          byte[p++] := "."
      else
        byte[p++] := "*" 
    byte[p]~

  else
    ' store formatted number
    p += width
    byte[p]~

    repeat width
      byte[--p] := n // 10 + "0"
      n /= 10
      if --dp == 0
        if decimal_chr
          byte[--p] := decimal_chr
        else
          byte[--p] := "."
      if n == 0 and dp < 0
        quit

    ' store sign      
    if single & $80000000
      byte[--p] := "-"
    elseif positive_chr
      byte[--p] := positive_chr
    ' leading blank fill
    repeat while p <> stringptr
      ++stringptr
'      byte[--p] := " "
      
'********************************
PUB SetPrecision(NumberOfDigits)
'********************************

  precision := NumberOfDigits

'********************************
PUB SetPositiveChr(PositiveChr)
'********************************

  positive_chr := PositiveChr

'********************************
PUB SetDecimalChr(DecimalChr)
'********************************

  decimal_chr := DecimalChr

'********************************
PUB SetSeparatorChrs(ThousandsChr, ThousandthsChr)
'********************************

  thousands_chr := ThousandsChr
  thousandths_chr := ThousandthsChr

'********************************
PRI Setup(single) : stringptr
'********************************

 'limit digits to 1..7
  if precision
    digits := precision #> 1 <# 7
  else
    digits := 7

  'initialize string pointer
  p := @float_string

  'add "-" if negative
  if single & $80000000
    byte[p++] := "-"
  'otherwise, add any positive lead character
  elseif positive_chr
    byte[p++] := positive_chr

  'clear sign and check for 0
  if single &= $7FFFFFFF

    'not 0, estimate exponent
    exponent := ((single << 1 >> 24 - 127) * 77) ~> 8
    
    'if very small, bias up
    if exponent < -32
      single := FMath.FMul(single, 1e13)
      exponent += result := 13
      
  'if 0, reset exponent and integer
  else
    exponent~
    integer~

  'set initial tens and clear zeros
  tens := teni[digits - 1]
  zeros~

  'return pointer to string
  stringptr := @float_string

'********************************
PRI AddDigits(leading) | i
'********************************

  'add leading digits
  repeat i := leading
    AddDigit
    'add any thousands separator between thousands
    if thousands_chr
      i--
      if i and not i // 3
        byte[p++] := thousands_chr
  'if trailing digits, add decimal character
  if digits
    AddDecimal
    'then add trailing digits
    repeat while digits
      'add any thousandths separator between thousandths
      if thousandths_chr
        if i and not i // 3
          byte[p++] := thousandths_chr
      i++
      AddDigit

'********************************
PRI AddDigit
'********************************

  'if leading zeros, add "0"
  if zeros
    byte[p++] := "0"
    zeros--
  'if more digits, add current digit and prepare next
  elseif digits
    byte[p++] := integer / tens + "0"
    integer //= tens
    tens /= 10
    digits--
  'if no more digits, add "0"
  else
    byte[p++] := "0"

'********************************
PRI AddDecimal
'********************************

  if decimal_chr
    byte[p++] := decimal_chr
  else
    byte[p++] := "."
                    
'********************************
DAT
'********************************

  teni  long  1, 10, 100, 1_000, 10_000, 100_000, 1_000_000, 10_000_000, 100_000_000, 1_000_000_000

        byte "yzafpnum"

{Test value section - remove in production}

  Px1   byte 40, 45, 45, 88, 100, 110, 120, 125, 130, 142, 150, 155, 130, 120, 113
        byte 100, 102, 104, 106, 112, 112, 113, 113, 111, 110, 99, 99, 100, 101, 103
        byte 103, 103, 106, 108, 115, 120, 130, 135, 136, 136, 136, 133, 132, 131, 131
        byte 129, 129, 127, 126, 124, 124, 124, 123, 123, 125, 125, 124, 124, 125, 120
        byte 110, 90,  87,  85,  82,  72,  65,  66,  68,  72,  80,  85,  90,  95,  100

  Px2   byte 61, 66, 65, 73, 88, 101, 109, 115, 120, 132, 139, 146, 135, 122, 115
        byte 104, 101, 102, 104, 108, 109, 110, 110, 110, 110, 98, 97, 99, 100, 103
        byte 102, 102, 104, 105, 108, 112, 118, 125, 131, 133, 134, 133, 133, 132, 132
        byte 127, 127, 124, 124, 121, 121, 121, 118, 119, 122, 123, 123, 124, 124, 121
        byte 113, 98,  91,  88,  84,  72,  62,  63,  64,  66,  72,  76,  84,  89,  94

  Px3   byte 66, 61, 83, 92, 107, 114, 126, 131, 138, 149, 153, 156, 138, 127, 120
        byte 106, 105, 105, 106, 116, 116, 115, 115, 115, 116, 110, 106, 105, 106, 108
        byte 108, 108, 111, 113, 119, 126, 137, 142, 144, 145, 145, 144, 144, 143, 143
        byte 138, 134, 131, 130, 129, 129, 128, 127, 126, 128, 129, 128, 128, 128, 126
        byte 118, 100,  97,  96,  90,  81,  72,  70,  71,  75,  83,  88,  94,  98,  104

  